#!/usr/bin/python3
# -*- coding: UTF-8 -*-

import json;
import subprocess;
import datetime;
import time;
import sys;
import os;


def GetJsonKey(Data):
	Return = [];
	for key, values in Data.items():
		Return.append(key);
	return Return;

def FilterModuleJson(ModuleJson):
	Key = GetJsonKey(ModuleJson);
	for i in range(len(Key)):
		ModuleJson[Key[i]]= list(set(ModuleJson[Key[i]]));
	return ModuleJson;

def GetModuleJson(JSON, Total_OID, Info, ModuleOID):
	ModuleJson = {};
	MIB_Module = "";
	for i in range(len(Total_OID)):
		if Total_OID[i].find(ModuleOID) != -1:
			ModuleSplit = JSON[Total_OID[i]]["Runtime"].split("_");
			OID = Total_OID[i].replace(ModuleOID, "");
			OID = OID.split(".")[0];
			Module = ModuleSplit[0] + "_" + ModuleSplit[1] + "_" + ModuleSplit[2];
			MIB_Module = Module + " OBJECT IDENTIFIER ::= { " + Info + " " + OID + " }";
			MIB_Module_DI = Module + "_DI OBJECT IDENTIFIER ::= { " + Module + " 1 }";
			MIB_Module_DO = Module + "_DO OBJECT IDENTIFIER ::= { " + Module + " 2 }";
			MIB_Module_AI = Module + "_AI OBJECT IDENTIFIER ::= { " + Module + " 3 }";
			MIB_Module_AO = Module + "_AO OBJECT IDENTIFIER ::= { " + Module + " 4 }";
			Key = GetJsonKey(ModuleJson);
			try:
				Key.index(MIB_Module);
			except Exception as e:
				ModuleJson[MIB_Module] = [];
			ModuleJson[MIB_Module].append(MIB_Module_DI);
			ModuleJson[MIB_Module].append(MIB_Module_DO);
			ModuleJson[MIB_Module].append(MIB_Module_AI);
			ModuleJson[MIB_Module].append(MIB_Module_AO);
	return ModuleJson;

def GenerateRtuInfo(JSON, Total_OID, File):
	ModuleJson = GetModuleJson(JSON, Total_OID, "rtuInfo", ".1.3.6.1.4.1.34321.50.2.");
	ModuleJson = FilterModuleJson(ModuleJson);
	Key = GetJsonKey(ModuleJson);
	for i in range(len(Key)):
		File.write(Key[i] + "\r\n");
		for j in range(len(ModuleJson[Key[i]])):
			File.write(ModuleJson[Key[i]][j] + "\r\n");
	File.write("\r\n\r\n\r\n");
	return;

def GenerateTcpInfo(JSON, Total_OID, File):
	ModuleJson = GetModuleJson(JSON, Total_OID, "tcpInfo", ".1.3.6.1.4.1.34321.50.3.");
	ModuleJson = FilterModuleJson(ModuleJson);
	Key = GetJsonKey(ModuleJson);
	for i in range(len(Key)):
		File.write(Key[i] + "\r\n");
		for j in range(len(ModuleJson[Key[i]])):
			File.write(ModuleJson[Key[i]][j] + "\r\n");
	File.write("\r\n\r\n\r\n");
	return;

def GenerateVariableInfo(JSON, Total_OID, File):
	for i in range(len(Total_OID)):
		Node = JSON[Total_OID[i]]["Runtime"].split(".")[0] + "." + JSON[Total_OID[i]]["Runtime"].split(".")[1];
		NodeNumber = Total_OID[i].split(".")[12];
		Tag = JSON[Total_OID[i]]["Runtime"];
		Describe = "";
		if Tag.find("_DI.") != -1:
			Describe = " OBJECT-TYPE\r\nSYNTAX INTEGER\r\nMAX-ACCESS read-only\r\nSTATUS current\r\nDESCRIPTION\r\n\"\" ::= { ";
		elif Tag.find("_DO.") != -1:
			Describe = " OBJECT-TYPE\r\nSYNTAX INTEGER\r\n{\r\noff(0),\r\non(1)\r\n}\r\nMAX-ACCESS read-write\r\nSTATUS current\r\nDESCRIPTION\r\n\"\" ::= { ";
		elif Tag.find("_AI.") != -1:
			Describe = " OBJECT-TYPE\r\nSYNTAX OCTET STRING\r\nMAX-ACCESS read-only\r\nSTATUS current\r\nDESCRIPTION\r\n\"\" ::= { ";
		elif Tag.find("_AO.") != -1:
			Describe = " OBJECT-TYPE\r\nSYNTAX OCTET STRING\r\nMAX-ACCESS read-write\r\nSTATUS current\r\nDESCRIPTION\r\n\"\" ::= { ";
		File.write(Tag + Describe + Node + " " + NodeNumber + " }" + "\r\n\r\n\r\n");
	return;	

def MergeFile(HeadMibPath, TemporaryPath, File):
	File = open(HeadMibPath, 'r');
	HeadMib = File.read();
	HeadMib = HeadMib.replace("END", "");
	File = open(TemporaryPath, 'r');
	TemporaryTxt = File.read();
	TemporaryTxt = TemporaryTxt.replace("No.", "No");
	TemporaryTxt = TemporaryTxt.replace(".", "_");
	TemporaryTxt = TemporaryTxt.replace("MRTU_", "mRtu_");
	TemporaryTxt = TemporaryTxt.replace("MTCP_", "mTcp_");
	TemporaryTxt = TemporaryTxt.replace("_", "");

	File = open(ModuleMibPath, 'w');
	File.write(HeadMib + TemporaryTxt);
	File.close();

	subprocess.check_output("rm -rf " + TemporaryPath, shell=True);
	return;

#===== main =====
FilePath = "/app/SNMP/File/Transform.json";
TemporaryPath = "/dev/shm/icpdas_snmp_temporary.txt";
HeadMibPath = "/app/SNMP/File/Head.mib";
ModuleMibPath = "/app/SNMP/File/IcpdasModule.mib";

File = open(FilePath, "r");
JSON = json.load(File);#輸入檔案物件 File
File.close();
Total_OID = GetJsonKey(JSON);
File = open(TemporaryPath, 'w');


GenerateRtuInfo(JSON, Total_OID, File);
GenerateTcpInfo(JSON, Total_OID, File);
GenerateVariableInfo(JSON, Total_OID, File);
File.write("END");
File.close();
MergeFile(HeadMibPath, TemporaryPath, File);