#!/bin/bash

led='/usr/sbin/iTalk/led'

blink() 
{
    while true; do
        $led -l2 -o1  # Turn on the LED
        sleep 0.5
        $led -l2 -o0  # Turn off the LED
        sleep 0.5
    done
}

UpdateService()
{
    for service_file in /lib/systemd/system/ua-*.service; do
        if [ ! -f "$1/Environment/Service/$(basename "$service_file")" ]; then
            systemctl disable "$(basename "$service_file")"
            rm -f "$service_file"
        fi
    done
    
}

DeleteServiceEnvironment()
{
   
    rm -rf /lib/systemd/system/ua-*;
}
KeepProjectFile()
{
	mkdir /tmp/Project/;
	cp -rf /app/Web/public/File /tmp/Project/;
	rm /tmp/Project/File/Version.txt;
	cp -rf /app/Web/cgi-bin /tmp/Project/;
	cp -rf /app/Certificates /tmp/Project/;
    cp -rf /app/DDNS/inadyn.conf /tmp/Project;
    cp -rf /app/MQTT/Broker/mosquitto.conf /tmp/Project/;
    cp -rf /app/runtime/project /tmp/Project/;
    cp -rf /app/Wireless/wvdial.conf /tmp/Project/;
    cp -rf /app/firewall/blocklist/rule_config.json /tmp/Project/;
    cp -rf /app/SNMP/File/snmpd.conf /tmp/Project/;
    cp -rf /app/SNMP/File/Transform.json /tmp/Project/;

	rm -rf /app/Web/public/File/TCP_Module/TcpIcpdasModule.json;
	rm -rf /app/Web/public/File/RTU_Module/RtuIcpdasModule.json;
}

RestoreProjectFiles()
{
	#2024/01/11
	cp -rf /tmp/Project/File /app/Web/public/;
	cp -rf /tmp/Project/cgi-bin /app/Web/;
	cp -rf /tmp/Project/Certificates /app/;
    cp -rf /tmp/Project/inadyn.conf /app/DDNS;
    cp -rf /tmp/Project/mosquitto.conf /app/MQTT/Broker/;
    cp -rf /tmp/Project/project /app/runtime/;
    cp -rf /tmp/Project/wvdial.conf /app/Wireless/;
    cp -rf /tmp/Project/rule_config.json /app/firewall/blocklist/;
    cp -rf /tmp/Project/snmpd.conf /app/SNMP/File/;
    cp -rf /tmp/Project/Transform.json /app/SNMP/File/;

	rm -rf /tmp/Project/;
	Sync;
}

WebUpdateCheckTarFileName()
{
    Total=`ls -l /mnt/SD/update/*.tar  | grep "^-" | wc -l`;
    if [ "$Total" != "1" ]; then
        expr -1;
        return;
    fi
    
    Path="/mnt/SD/update/";
    TarName=`ls "$Path"ua-arm64*.tar`;
    if [ "$TarName" = "" ]; then
        expr 0;
        return;
    fi

    expr 1;
}

RenameAPPFolder()
{
	rm -rf /app_old;
	mv /app /app_old;
	Sync;
}

WriteStringToWebLog()
{
	echo "$1" > /tmp/WebUpdate.log;
}

WebUpdateDecompressedFile()
{
    
	tar -xf /mnt/SD/update/ua-arm64*.tar -C /.;
	if [ $? -ne 0 ]; then
		WriteStringToWebLog "Step2: Decompressed Firmware Fail.";
		rm -rf /command*.sh;
        rm -rf /command*.sh;
        rm -rf /function.sh;
        rm -rf /Environment;
        rm -rf /app.tar.gz ;
        rm -rf /rotarySwitch.sh;      
		mv /app_old /app;
		rm -rf /mnt/SD/update;
		mv /mnt/SD/factory_old /factory;
		Beep 4 0.25 0.1;
		exit 0;
	else
	(
		tar -zxf /app.tar.gz -C /. 
		Sync;
	) ||(WriteStringToWebLog "Step2: Decompressed Firmware Fail.";\
		rm -rf /app; mv /app_old /app;mv /mnt/SD/factory_old /factory;\
		rm -rf /mnt/SD/update;Beep 4 0.25 0.1;exit 0;)
	fi	
    Sync;
	
}

WebUpdateComparisonAPPFile()
{
    Return=0;
    ComparisonMessage=`diff -r /app/ /mnt/SD/update/app/`;
    if [ "$ComparisonMessage" != "" ]; then
	Return=-1;
    fi
    expr $Return;
}
WebUpdateDeleteDecompressedFile()
{
    rm -rf /app.tar.gz;

}
DeleteWebRedundantFile()
{
    rm -rf /mnt/SD/update/command*.sh;
    rm -rf /mnt/SD/update/function.sh;
    rm -rf /mnt/SD/update/rotarySwitch.sh;
    rm -rf /mnt/SD/update/Environment;
    rm -rf /mnt/SD/update/app.tar.gz;
    rm -rf /mnt/SD/update/ua-arm64*.tar;

    rm -rf /ua-arm64*.tar;
    rm -rf /command*.sh;
    rm -rf /function.sh;
    rm -rf /Environment;
    rm -rf /app.tar.gz ;
    rm -rf /rotarySwitch.sh;

}

DeleteOldAPPFolder()
{
	rm -rf /app_old;
	Sync;
}

RenameFactoryFolder()
{
	rm -rf /mnt/SD/factory_old;
	mv /factory /mnt/SD/factory_old;
}

CreatFactoryEnvironment()
{
	mkdir /factory;
	cp /mnt/SD/update/ua-arm64*.tar /factory/.;
	if [ $? -ne 0 ]; then
	{
	  rm -rf /factory;
	  mv /mnt/SD/factory_old /factory;
	  WriteStringToWebLog "Step3: Update Factory Firmware Fail.";
	  Beep 4 0.25 0.1;
	}
	else
	WriteStringToWebLog "Step3: Update Factory Firmware Successful.";
	fi
	
}

DeleteOldFactoryFolder()
{
	rm -rf /mnt/SD/factory_old;
}

Beep()
{
  FREQ_Hz=$(awk -v x=$2 'BEGIN {printf "%d",1/x}')
  LEN_ms=$(awk -v x=$2 'BEGIN {printf "%d",x*1000}')
  DELAY_ms=$(awk -v x=$3 'BEGIN {printf "%d",x*1000}')
  for(( i=0; i<$1; i=i+1))
  do
     beep -f $FREQ_Hz -l $LEN_ms -r 1 -D $DELAY_ms
  done
}

ClearApp()
{
    killall "MainProgram.sh" >& /dev/null;
    killall "MainProgram" >& /dev/null;
    killall "mosquitto" >& /dev/null;
    killall "inadyn" >& /dev/null;
    killall "ua-backstageService" >& /dev/null;
    systemctl stop ua-runtime.service;
    systemctl stop ua-broker.service;
    systemctl stop ua-backstageService.service;
    systemctl stop ua-snmpd.service;
    systemctl stop a-iftttConditionTrigger.service;
}

RemoveApp()
{
    rm -rf /app;
    find /lib/systemd/system/ua-* ! -name 'ua-rotarySwitch.service' -type f -exec rm -f {} +
}

RemoveFactory()
{
    rm -rf /factory;
}

CreateEnvironment()
{
    mkdir /factory;
    Sync;
}

UsbCheckTarFileName()
{
    Total=`ls -l /mnt/USB/*.tar  | grep "^-" | wc -l`;
    if [ "$Total" != "1" ]; then
        expr -1;
        return;
    fi
    
    Path="/mnt/USB/";
    TarName=`ls "$Path"ua-arm64*.tar`;
    if [ "$TarName" = "" ]; then
        expr 0;
        return;
    fi

    expr 1;
}


CheckTarFileName()
{
    Total=`ls -l /mnt/SD/*.tar  | grep "^-" | wc -l`;
    if [ "$Total" != "1" ]; then
        expr -1;
        return;
    fi
    
    Path="/mnt/SD/";
    TarName=`ls "$Path"ua-arm64*.tar`;
    if [ "$TarName" = "" ]; then
        expr 0;
        return;
    fi

    expr 1;
}

UsbCopyInstallFileToEnvironment()
{
    cp -rf /mnt/USB/ua-arm64*.tar /.;
    Sync;
    rm -rf /factory/*;
    cp -rf /mnt/USB/ua-arm64*.tar /factory;
    Sync;
}

CopyInstallFileToEnvironment()
{
    cp -rf /mnt/SD/ua-arm64*.tar /.;
    Sync;
    rm -rf /factory/*;
    cp -rf /mnt/SD/ua-arm64*.tar /factory;
    Sync;
}

CopyFactoryFileToEnvironment()
{
    cp -rf /factory/ua-arm64*.tar /.;
    Sync;
}

ComparisonFactoryTarFile()
{
    Return=0;
    TarName1=`ls /factory/*.tar`;
    TarName2=`ls /*.tar`;
    ComparisonMessage=`diff $TarName1 $TarName2`;
    if [ "$ComparisonMessage" != "" ]; then
	Return=-1;
    fi
    expr $Return;
}

UsbComparisonTarFile()
{
    Return=0;
    TarName1=`ls /mnt/USB/*.tar`;
    TarName2=`ls /*.tar`;
    ComparisonMessage=`diff $TarName1 $TarName2`;
    if [ "$ComparisonMessage" != "" ]; then
	Return=-1;
    fi
    expr $Return;
}

ComparisonTarFile()
{
    Return=0;
    TarName1=`ls /mnt/SD/*.tar`;
    TarName2=`ls /*.tar`;
    ComparisonMessage=`diff $TarName1 $TarName2`;
    if [ "$ComparisonMessage" != "" ]; then
	Return=-1;
    fi
    expr $Return;
}

DecompressedFile()
{
    tar -xf /ua-arm64*.tar -C /.;
    Sync;
    tar -zxf /app.tar.gz -C /.;
    Sync;
    
}
UsbCopyEnvironment()
{
#Common
    CopyCommonFile;
#Wireless
    #CopyWirelessFile;
}

CopyEnvironment()
{
#Common
    CopyCommonFile;
#Wireless
    #CopyWirelessFile;
}

CopyCommonFile()
{
	chmod 644 /Environment/Service/*.service;
    cp -rf /Environment/Service/*.service /lib/systemd/system/;
	
	cp -rf /rotarySwitch.sh /icpdas/init/;
   	chmod 755 /icpdas/init/rotarySwitch.sh >& /dev/null;
}

CopyWirelessFile()
{
    Wireless_3G=`lsusb -d 05c6:`;
    Wireless_4G=`lsusb -d 2c7c:`;
   
}

UsbSetEnvironment()
{ 
    systemctl enable ua-searchIP.service;
    systemctl enable ua-rotarySwitch.service;
    systemctl enable ua-web.service;
    systemctl enable ua-iftttConditionTrigger.service;
    systemctl enable ua-daemon.service;
    systemctl enable ua-backstageService.service;
    systemctl enable ua-dynamic-blocklist.service;
}

SetEnvironment()
{
   
    systemctl enable ua-searchIP.service;
    systemctl enable ua-rotarySwitch.service;
    systemctl enable ua-runtime.service;
    systemctl enable ua-broker.service;
    systemctl enable ua-web.service;
    systemctl enable ua-iftttConditionTrigger.service;
    systemctl enable ua-daemon.service;
    systemctl enable ua-backstageService.service;
    systemctl enable ua-dynamic-blocklist.service;
    systemctl disable ua-ddns.service;
    systemctl disable ua-snmpd.service;
}

SetStaticIP()
{
    Return=0;
    _IP1=`ipv4set eth0 static 192.168.255.1 255.255.0.0 192.168.1.1 2>&1`;
    if [[ $_IP1 != *"applied successfully"* ]]; then
          Return=-1;
    fi
    _IP2=`ipv4set eth1 static 10.0.0.1 255.255.0.0 10.168.1.1 2>&1`;
    if [[ $_IP2 != *"applied successfully"* ]]; then
          Return=-1;
    fi
    expr $Return;
}

ChmodFile()
{
    chown -R root:root /app
    chmod 755 /app/* -R >& /dev/null;
    #chmod 755 /app/MQTT/Broker/* >& /dev/null;
    #chmod 755 /app/runtime/MainProgram >& /dev/null;
    #chmod 755 /rotarySwitch.sh >& /dev/null;
    #chmod 755 /app/SearchIP/* >& /dev/null;
    #chmod 755 /app/Web/* >& /dev/null;
    #chmod 755 /app/DBus/* >& /dev/null;
    #chmod 755 /app/ua-backstageService/bin/* >& /dev/null;
    #Wireless
    #chmod 755 /app/Wireless/* >& /dev/null;
    #chmod 755 /etc/ppp/ip-up.d/PPP0route >& /dev/null;
    #eeprom
    #chmod 755 /app/Eeprom/* >& /dev/null;
    #database
    #chmod 755 /app/MysqlConnecti/bin/* >& /dev/null;
    #chmod 755 /app/MySqlConnecti/bin/* >& /dev/null;

}

UsbDeleteResidualFile()
{
    rm -rf /mnt/USB/command*.sh;
    rm -rf /mnt/USB/function.sh;
    rm -rf /mnt/USB/rotarySwitch.sh;
    rm -rf /mnt/USB/Environment;
    rm -rf /mnt/USB/app.tar.gz;

    rm -rf /ua-arm64*.tar;
    rm -rf /command*.sh;
    rm -rf /function.sh;
    rm -rf /Environment;
    rm -rf /rotarySwitch.sh;
    rm -rf /app.tar.gz;
    
}

DeleteResidualFile()
{
    rm -rf /mnt/SD/command*.sh;
    rm -rf /mnt/SD/function.sh;
    rm -rf /mnt/SD/rotarySwitch.sh;
    rm -rf /mnt/SD/Environment;
    rm -rf /mnt/SD/app.tar.gz;

    rm -rf /ua-arm64*.tar;
    rm -rf /command*.sh;
    rm -rf /function.sh;
    rm -rf /Environment;
    rm -rf /rotarySwitch.sh;
    rm -rf /app.tar.gz;
}


DeleteFactoryResidualFile()
{
    rm -rf /factory/command*.sh;
    rm -rf /factory/function.sh;
    rm -rf /factory/rotarySwitch.sh;
    rm -rf /factory/Environment;
    rm -rf /factory/app.tar.gz;
}

Sync()
{
    sync;sync;sync;
}

ClearEeprom()
{
 
    /app/Eeprom/eep -w 0 "" -s >& /dev/null;

}

SetEeprom()
{
    #ClearEeprom;
    Message=$(date +%Y/%m/%d-%H:%M:%S)" $1";
    /app/Eeprom/eep -w 0 "$Message" -s;
}



ShowStartInstall()
{
    printf "\E[0;32;40m";
    echo "########## Start Install ##########";
    printf "\E[0m";
}

ShowNetworkConnectionError()
{
    printf "\E[0;31;40m";
    echo "xxxxxxxxxx Please check your network connection xxxxxxxxxx";
    printf "\E[0m";
}

ShowCheckTarFileNameError()
{
    printf "\E[0;31;40m";
    echo "xxxxxxxxxx Check tar file name xxxxxxxxxx";
    printf "\E[0m";

}

ShowComparisonTarFileError()
{
    printf "\E[0;31;40m";
    echo "xxxxxxxxxx Comparison file error xxxxxxxxxx";
    printf "\E[0m";
}

ShowRuntimePackageError()
{
    printf "\E[0;31;40m";
    echo "xxxxxxxxxx Runtime package file is missing xxxxxxxxxx";
    printf "\E[0m";
}

ShowInstallSuccess()
{
    printf "\E[0;32;40m";
    echo "########## Install success ##########";
    printf "\E[0m";
  
}

ChangeComPortDeviceSetting()
{
    printf "\E[0;32;40m";
    echo "########## Install success ##########";
    printf "\E[0m";
}

